;;; LCN.LSP Construction line routines to emulate Prime Medusa LCN's.
;;; Usage (lcn "VERT") (lcn "HORIZ") (lcn "VTOFFSET") (lcn "HZOFFSET")
;;; (lcn "1PTANG") (lcn "X") (lcn "ERASE")
;;; 01-07-93 Pete Olinger (Compuserve eMail 72644,1725)

;;; Draws solid tan (color 10) lines on layer CONST from one side of the
;;; current window to the other. Osnap selection may be used to pick requested
;;; points. Offset VERTICAL and HORIZONTAL lines request an offset value; enter
;;; the number as units (architectural ' '' not allowed). A positive value
;;; offsets to the right/top and negative to the left/bottom respectively. The
;;; 1PTANG requests an angle - any positive decimal angle is allowed. The "X"
;;; option draws two lines in an "X", filling the window, intersecting at the
;;; selected point. ERASE erases all entities on LAYER CONST.

;;; LINES FOR PULL DOWN MENU:
;;; ***POP10
;;; [MISC. STUFF]
;;; [~--]
;;; [CONSTRUCTION LINES>]^C^C(LOAD "LCN") $P10=P10CONST;$P10=*;
;;; .
;;; .
;;; .
;;; **P10CONST
;;; [CONSTRUCT LINES]
;;; [~--]
;;; [VERTICAL]^C^C(LCN "VERT")
;;; [HORIZONTAL]^C^C(LCN "HORIZ")
;;; [VERT OFFSET]^C^C(LCN "VTOFFSET")
;;; [HORIZ OFFSET]^C^C(LCN "HZOFFSET")
;;; [1 POINT ANGLE]^C^C(LCN "1PTANG")
;;; [X]^C^C(LCN "X")
;;; [ERASE]^C^C(LCN "ERASE")
;;; [~--]
;;; [MISC. STUFF >]^C^C$P10= $P10=*

(defun reduce (tmp)                     ;reduce angles >= 360 degrees
 (while (> tmp (* pi 2.0))
  (setq tmp (- tmp (* pi 2.0)))
 )
 (setq ang tmp)
)
(defun tan (ang)                        ;define tangent function
 (/ (sin ang)(cos ang))
)
(defun layset ()                        ;save current layer and make CONST
 (setq slay (getvar "CLAYER"))          ;the new layer
 (if (tblsearch "LAYER" "CONST")
  (command "LAYER" "S" "CONST" "")
  (command "LAYER" "M" "CONST" "L" "0" "" "C" "10" "" "");uses linetype 0
 )
)                                       ;get current screen size
(defun getvs ()                         ;set min/max values
 (setq xfac (car (getvar "SCREENSIZE"))
       yfac (cadr (getvar "SCREENSIZE"))
       vsmaxx (+(car(getvar "VIEWCTR"))(/(*(getvar "VIEWSIZE")(/ xfac yfac)) 2))
       vsmaxy (+(cadr(getvar "VIEWCTR"))(/(getvar "VIEWSIZE")2))
       vsminx (-(car(getvar "VIEWCTR"))(/(*(getvar "VIEWSIZE")(/ xfac yfac)) 2))
       vsminy (-(cadr(getvar "VIEWCTR"))(/(getvar "VIEWSIZE")2))
 )
)
(defun pntone ()                        ;select point and seperate x, y
 (setq inp (getpoint "\nPick point ")
       inpx (car inp)
       inpy (cadr inp)
 )
)
(defun vlcn ()                          ;draw vertical line
 (setq pt1 (list inpx vsminy)
       pt2 (list inpx vsmaxy)
 )
 (command "LINE" pt1 pt2 "")
 (princ)
)
(defun hlcn ()                          ;draw horizontal line
 (setq pt1 (list vsminx inpy)
       pt2 (list vsmaxx inpy)
 )
 (command "LINE" pt1 pt2 "")
 (princ)
)
(defun volcn ()                         ;draw vertical offset line
 (setq inpx (+ inpx (getreal "\nEnter offset "))
       pt1 (list inpx vsminy)
       pt2 (list inpx vsmaxy)
 )
 (command "LINE" pt1 pt2 "")
 (princ)
)
(defun holcn ()                         ;draw horizontal offset line
 (setq inpy (+ inpy (getreal "\nEnter offset "))
       pt1 (list vsminx inpy)
       pt2 (list vsmaxx inpy)
 )
 (command "LINE" pt1 pt2 "")
 (princ)
)
(defun eraselcn ()                      ;erase everything on layer CONST
 (command "SELECT" (ssget "X" (list (cons 8 "CONST"))) "")
 (command "ERASE" "P" "")
 (princ)
)
(defun quad1 ()                         ;1st and 3rd quadrant angle
 (setq x (- inpx vsminx)
       y (* x (tan ang))
       x3 vsminx
       y3 (- inpy y)
 )
 (if (< y3 vsminy)
  (setq y (- inpy vsminy)
        x (/ y (tan ang))
        y3 vsminy
        x3 (- inpx x)
  )
 )
 (setq x (- vsmaxx inpx)
       y (* x (tan ang))
       x2 vsmaxx
       y2 (+ inpy y)
 )
 (if (> y2 vsmaxy)
  (setq y (- vsmaxy inpy)
        x (/ y (tan ang))
        x2 (+ inpx x)
        y2 vsmaxy
  )
 )
)
(defun quad2 ()                         ;2nd and 4th quandrant angle
 (setq x (- inpx vsminx)
       y (* x (tan ang))
       x2 vsminx
       y2 (+ inpy y)
 )
 (if (> y2 vsmaxy)
  (setq y (- vsmaxy inpy)
        x (/ y (tan ang))
        y2 vsmaxy
        x2 (- inpx x)
  )
 )
 (setq x (- vsmaxx inpx)
       y (* x (tan ang))
       x3 vsmaxx
       y3 (- inpy y)
 )
 (if (< y3 vsminy)
  (setq y (- inpy vsminy)
        x (/ y (tan ang))
        x3 (+ inpx x)
        y3 vsminy
  )
 )
)
(defun drwang ()                        ;draw angled line
 (setq pt1 (list x2 y2)
       pt2 (list x3 y3)
 )
 (command "LINE" pt1 pt2 "")
)
(defun 1ptang (ang)                     ;angled line function
 (reduce ang)                           ;return with angle < 360 degrees
 (if (and (< ang (* pi 0.5))(> ang 0.0));quad1
  (progn
   (quad1)
   (drwang)
  )
 )
 (if (and (> ang (* pi 0.5))(< ang pi)) ;quad2
  (progn
   (setq ang (- pi ang))
   (quad2)
   (drwang)
  )
 )
 (if (and (> ang pi)(< ang (* pi 1.5))) ;quad3
  (progn
   (setq ang (- ang pi))
   (quad1)                              ;use quad1
   (drwang)
  )
 )
 (if (and (> ang (* pi 1.5))(< ang (* pi 2.0)));quad4
  (progn
   (setq ang (- (* pi 2.0) ang))
   (quad2)                              ;use quad2
   (drwang)
  )
 )
 (if (or (= ang (* pi 0.5))(= ang (* pi 1.5)));vertical (90 or 270 degrees)
  (vlcn)
 )
 (if (or (= ang pi)(= ang (* 2.0 pi)))  ;horizontal (180 or 0 degrees)
  (hlcn)
 )
)
(defun lcn (opt / ang slay xfac yfac vsmaxx vsmaxy vsminx vsminy inp inpx inpy
                  pt1 pt2 x y x2 y2 x3 y3);main function
 (layset)
 (getvs)
 (cond                                  ;what to do
  ((= opt "VERT")(pntone)(vlcn))
  ((= opt "HORIZ")(pntone)(hlcn))
  ((= opt "VTOFFSET")(pntone)(volcn))
  ((= opt "HZOFFSET")(pntone)(holcn))
  ((= opt "1PTANG")
    (pntone)
    (setq ang (* pi (/ (getreal "\nEnter angle.. ") 180.0)))
    (1ptang ang)
  )
  ((= opt "X")
    (pntone)
    (setq ang (* pi 0.25))
    (1ptang ang)
    (setq ang (* pi 0.75))
    (1ptang ang)
  )
  ((= opt "2PTANG") nil)                ;challenge to draw angled line by 2 pts
  ((= opt "ERASE")(eraselcn))
  (T nil)
 )
 (command "LAYER" "S" slay "")          ;return to saved LAYER
 (princ)
)
